library flutter_compute_tree;

import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/scheduler.dart';
import 'package:flutter/widgets.dart';

import 'package:matches/matches.dart';
export 'package:matches/matches.dart';

part 'compute_tree_node.dart';
part 'compute_tree_widget.dart';
part 'node_builder.dart';
part 'unit.dart';
part 'unit_entry.dart';
part 'unit_key.dart';
part 'units.dart';
part 'untis/invoke.dart';
part 'untis/invoke_async.dart';
part 'untis/memo.dart';
part 'untis/ref.dart';
part 'untis/stream_subscirption.dart';
part 'untis/ref_subscription.dart';
part 'untis/trigger.dart';

const dynamic _constDep = Object();

extension RefExtension on CTNode {
  Ref<T> ref<T, K extends Object>(T Function() factory, [K? dep]) {
    dep ??= _constDep;
    return _units.next(() => _RefUnit<T, K>(factory), dep as K);
  }
}

extension MemoExtension on CTNode {
  T memo<T>(T value) {
    return _units.next<T, Object>(() => _MemoUnit(value), const Object());
  }
}

class Invoke {
  final Units _units;
  const Invoke._(this._units);

  T call<T, K extends Object>(T Function() fn, [K? dep]) {
    dep ??= _constDep;
    return _units.next<T, K>(() => _InvokeUnit(fn), dep as K);
  }

  T deferred<T, K extends Object>(
    T Function() fn, {
    K? dep,
    required Future<void> until,
    required T initialValue,
  }) {
    dep ??= _constDep;

    return _units.next<T, K>(
      () => _InvokeUnit(fn, until, initialValue),
      dep as K,
    );
  }

  Result<T> async<T, K extends Object, R>(
    Future<T> Function() fn, {
    void Function()? abort,
    Future<void>? deferredUntil,
    K? dep,
  }) {
    dep ??= _constDep;
    return _units.next<Result<T>, K>(
      () => _InvokeAsyncUnit(fn, abort, deferredUntil),
      dep as K,
    );
  }

  Result<T> afterLayout<T, K extends Object>(
    T Function() fn, {
    K? dep,
  }) {
    dep ??= _constDep;

    final completer = _units._owner.ref(() => Completer<T>(), dep);

    _units._owner.invoke(() {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        completer.value.complete(fn());
      });
    }, dep);

    return _units.next<Result<T>, K>(
      () => _InvokeAsyncUnit(
        () => completer.value.future,
        null,
        completer.value.future,
      ),
      dep as K,
    );
  }
}

extension InvokeExtensioon on CTNode {
  Invoke get invoke => Invoke._(_units);
}

extension TriggerExtension on CTNode {
  Trigger<T> trigger<T extends Object>() {
    return _units.next<Trigger<T>, Object>(
      () => _TriggerUnit<T>(this),
      const Object(),
    );
  }
}
