part of '../flutter_compute_tree.dart';

class _InvokeAsyncUnit<T, K> extends Unit<Result<T>, K> {
  final Future<T> Function() _fn;
  final void Function()? _abort;
  final Future<void>? _deferredUntil;

  late Result<T> _value;

  static final Future<void> _defaultDeferred = SynchronousFuture(null);

  _InvokeAsyncUnit(this._fn, [this._abort, this._deferredUntil]);

  @override
  void init(CTNode node, K dep) {
    super.init(node, dep);
    _value = Pending<T>();

    (_deferredUntil ?? _defaultDeferred).whenComplete(_load);
  }

  void _load() {
    _value = Loading<T>();
    notify();

    _fn().then((value) {
      _value = Success(value);
      notify();
    }).catchError((e, s) {
      _value = Failure(e, s);
      notify();
    });
  }

  // TODO: test if this could be dropped
  @override
  Unit<Result<T>, K> update(
    CTNode node,
    K oldDep,
    K newDep,
    _InvokeAsyncUnit<T, K> newUnit,
  ) {
    super.update(node, oldDep, newDep, newUnit);

    if (oldDep == newDep) {
      return this;
    }

    _abort?.call();
    (newUnit._deferredUntil ?? _defaultDeferred).whenComplete(_load);

    return this;
  }

  @override
  void dispose() {
    _abort?.call();
    super.dispose();
  }

  @override
  Result<T> unwrap() {
    return _value;
  }
}
